@@ -42,13 +42,8 @@
 #include "qquickanimatorjob_p.h"
 #include "qquickanimator_p.h"
 #include "qquickanimator_p_p.h"
-#include <private/qquickwindow_p.h>
 #include <private/qquickitem_p.h>
-#if QT_CONFIG(quick_shadereffect) && QT_CONFIG(opengl)
-# include <private/qquickopenglshadereffectnode_p.h>
-# include <private/qquickopenglshadereffect_p.h>
-# include <private/qquickshadereffect_p.h>
-#endif
+#include <private/qquickshadereffect_p.h>
 #include <private/qanimationgroupjob_p.h>
 
 #include <qcoreapplication.h>
@@ -123,6 +118,11 @@ QQuickAnimatorProxyJob::QQuickAnimatorProxyJob(QAbstractAnimationJob *job, QObje
     }
 }
 
+void QQuickAnimatorProxyJob::updateLoopCount(int loopCount)
+{
+    m_job->setLoopCount(loopCount);
+}
+
 QQuickAnimatorProxyJob::~QQuickAnimatorProxyJob()
 {
     if (m_job && m_controller)
@@ -143,6 +143,10 @@ void QQuickAnimatorProxyJob::updateCurrentTime(int)
     if (m_internalState != State_Running)
         return;
 
+    // Copy current loop number from the job
+    // we could make currentLoop() virtual but it would be less efficient
+    m_currentLoop = m_job->currentLoop();
+
     // A proxy which is being ticked should be associated with a window, (see
     // setWindow() below). If we get here when there is no more controller we
     // have a problem.
@@ -184,7 +188,7 @@ void QQuickAnimatorProxyJob::updateState(QAbstractAnimationJob::State newState,
 
 void QQuickAnimatorProxyJob::debugAnimation(QDebug d) const
 {
-    d << "QuickAnimatorProxyJob("<< hex << (const void *) this << dec
+    d << "QuickAnimatorProxyJob("<< Qt::hex << (const void *) this << Qt::dec
       << "state:" << state() << "duration:" << duration()
       << "proxying: (" << job() << ')';
 }
@@ -207,7 +211,7 @@ void QQuickAnimatorProxyJob::setWindow(QQuickWindow *window)
         stop();
 
     } else if (!m_controller && m_job) {
-        m_controller = QQuickWindowPrivate::get(window)->animationController;
+        m_controller = QQuickWindowPrivate::get(window)->animationController.get();
         if (window->isSceneGraphInitialized())
             readyToAnimate();
         else
@@ -266,7 +270,7 @@ QQuickAnimatorJob::QQuickAnimatorJob()
 
 void QQuickAnimatorJob::debugAnimation(QDebug d) const
 {
-    d << "QuickAnimatorJob(" << hex << (const void *) this << dec
+    d << "QuickAnimatorJob(" << Qt::hex << (const void *) this << Qt::dec
       << ") state:" << state() << "duration:" << duration()
       << "target:" << m_target << "value:" << m_value;
 }
@@ -431,9 +435,6 @@ void QQuickXAnimatorJob::writeBack()
 
 void QQuickXAnimatorJob::updateCurrentTime(int time)
 {
-#if QT_CONFIG(opengl)
-    Q_ASSERT(!m_controller || !m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
-#endif
     if (!m_helper)
         return;
 
@@ -450,9 +451,6 @@ void QQuickYAnimatorJob::writeBack()
 
 void QQuickYAnimatorJob::updateCurrentTime(int time)
 {
-#if QT_CONFIG(opengl)
-    Q_ASSERT(!m_controller || !m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
-#endif
     if (!m_helper)
         return;
 
@@ -469,9 +467,6 @@ void QQuickScaleAnimatorJob::writeBack()
 
 void QQuickScaleAnimatorJob::updateCurrentTime(int time)
 {
-#if QT_CONFIG(opengl)
-    Q_ASSERT(!m_controller || !m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
-#endif
     if (!m_helper)
         return;
 
@@ -492,9 +487,6 @@ extern QVariant _q_interpolateCounterclockwiseRotation(qreal &f, qreal &t, qreal
 
 void QQuickRotationAnimatorJob::updateCurrentTime(int time)
 {
-#if QT_CONFIG(opengl)
-    Q_ASSERT(!m_controller || !m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
-#endif
     if (!m_helper)
         return;
 
@@ -582,6 +574,7 @@ void QQuickOpacityAnimatorJob::postSync()
         }
 
         d->extra.value().opacityNode = m_opacityNode;
+        updateCurrentTime(0);
     }
     Q_ASSERT(m_opacityNode);
 }
@@ -599,10 +592,6 @@ void QQuickOpacityAnimatorJob::writeBack()
 
 void QQuickOpacityAnimatorJob::updateCurrentTime(int time)
 {
-#if QT_CONFIG(opengl)
-    Q_ASSERT(!m_controller || !m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
-#endif
-
     if (!m_opacityNode)
         return;
 
@@ -610,28 +599,31 @@ void QQuickOpacityAnimatorJob::updateCurrentTime(int time)
     m_opacityNode->setOpacity(m_value);
 }
 
-
-#if QT_CONFIG(quick_shadereffect) && QT_CONFIG(opengl)
 QQuickUniformAnimatorJob::QQuickUniformAnimatorJob()
-    : m_node(nullptr)
-    , m_uniformIndex(-1)
-    , m_uniformType(-1)
 {
     m_isUniform = true;
 }
 
 void QQuickUniformAnimatorJob::setTarget(QQuickItem *target)
 {
-    QQuickShaderEffect* effect = qobject_cast<QQuickShaderEffect*>(target);
-    if (effect && effect->isOpenGLShaderEffect())
+    // Check target is of expected type
+    if (qobject_cast<QQuickShaderEffect *>(target) != nullptr)
         m_target = target;
 }
 
-void QQuickUniformAnimatorJob::invalidate()
+void QQuickUniformAnimatorJob::updateCurrentTime(int time)
 {
-    m_node = nullptr;
-    m_uniformIndex = -1;
-    m_uniformType = -1;
+    if (!m_effect)
+        return;
+
+    m_value = m_from + (m_to - m_from) * progress(time);
+    m_effect->updateUniformValue(m_uniform, m_value);
+}
+
+void QQuickUniformAnimatorJob::writeBack()
+{
+    if (m_target)
+        m_target->setProperty(m_uniform, value());
 }
 
 void QQuickUniformAnimatorJob::postSync()
@@ -641,53 +633,14 @@ void QQuickUniformAnimatorJob::postSync()
         return;
     }
 
-    m_node = static_cast<QQuickOpenGLShaderEffectNode *>(QQuickItemPrivate::get(m_target)->paintNode);
-
-    if (m_node && m_uniformIndex == -1 && m_uniformType == -1) {
-        QQuickOpenGLShaderEffectMaterial *material =
-                static_cast<QQuickOpenGLShaderEffectMaterial *>(m_node->material());
-        bool found = false;
-        for (int t=0; !found && t<QQuickOpenGLShaderEffectMaterialKey::ShaderTypeCount; ++t) {
-            const QVector<QQuickOpenGLShaderEffectMaterial::UniformData> &uniforms = material->uniforms[t];
-            for (int i=0; i<uniforms.size(); ++i) {
-                if (uniforms.at(i).name == m_uniform) {
-                    m_uniformIndex = i;
-                    m_uniformType = t;
-                    found = true;
-                    break;
-                }
-            }
-        }
-    }
-
+    m_effect = qobject_cast<QQuickShaderEffect *>(m_target);
 }
 
-void QQuickUniformAnimatorJob::updateCurrentTime(int time)
+void QQuickUniformAnimatorJob::invalidate()
 {
-    if (!m_controller)
-        return;
-
-    if (!m_node || m_uniformIndex == -1 || m_uniformType == -1)
-        return;
-
-    m_value = m_from + (m_to - m_from) * progress(time);
-
-    QQuickOpenGLShaderEffectMaterial *material =
-            static_cast<QQuickOpenGLShaderEffectMaterial *>(m_node->material());
-    material->uniforms[m_uniformType][m_uniformIndex].value = m_value;
-    // As we're not touching the nodes, we need to explicitly mark it dirty.
-    // Otherwise, the renderer will abort repainting if this was the only
-    // change in the graph currently rendering.
-    m_node->markDirty(QSGNode::DirtyMaterial);
+    m_effect = nullptr;
 }
 
-void QQuickUniformAnimatorJob::writeBack()
-{
-    if (m_target)
-        m_target->setProperty(m_uniform, value());
-}
-#endif
-
 QT_END_NAMESPACE
 
 #include "moc_qquickanimatorjob_p.cpp"
