@@ -36,9 +36,12 @@
 #include <QSysInfo>
 #include <QLoggingCategory>
 #include <private/qqmlcomponent_p.h>
+#include <private/qqmlscriptdata_p.h>
 #include <qtranslator.h>
 
-class tst_qmlcachegen: public QObject
+#include "../../shared/util.h"
+
+class tst_qmlcachegen: public QQmlDataTest
 {
     Q_OBJECT
 
@@ -52,6 +55,8 @@ private slots:
     void aheadOfTimeCompilation();
     void functionExpressions();
     void versionChecksForAheadOfTimeUnits();
+    void retainedResources();
+    void skippedResources();
 
     void workerScripts();
 
@@ -61,10 +66,18 @@ private slots:
     void qrcScriptImport();
     void fsScriptImport();
     void moduleScriptImport();
+    void esModulesViaQJSEngine();
 
     void enums();
 
     void sourceFileIndices();
+
+    void reproducibleCache_data();
+    void reproducibleCache();
+
+    void parameterAdjustment();
+    void inlineComponent();
+    void posthocRequired();
 };
 
 // A wrapper around QQmlComponent to ensure the temporary reference counts
@@ -94,7 +107,7 @@ static bool generateCache(const QString &qmlFileName, QByteArray *capturedStderr
     QProcess proc;
     if (capturedStderr == nullptr)
         proc.setProcessChannelMode(QProcess::ForwardedChannels);
-    proc.setProgram(QLibraryInfo::location(QLibraryInfo::BinariesPath) + QDir::separator() + QLatin1String("qmlcachegen"));
+    proc.setProgram(QLibraryInfo::path(QLibraryInfo::BinariesPath) + QDir::separator() + QLatin1String("qmlcachegen"));
     proc.setArguments(QStringList() << qmlFileName);
     proc.start();
     if (!proc.waitForFinished())
@@ -118,6 +131,7 @@ void tst_qmlcachegen::initTestCase()
     if (!cacheDir.isEmpty())
         //QDir(cacheDir).removeRecursively();
         qDebug() << cacheDir;
+    QQmlDataTest::initTestCase();
 }
 
 void tst_qmlcachegen::loadGeneratedFile()
@@ -352,20 +366,23 @@ static QQmlPrivate::CachedQmlUnit *temporaryModifiedCachedUnit = nullptr;
 
 void tst_qmlcachegen::versionChecksForAheadOfTimeUnits()
 {
-    QVERIFY(QFile::exists(":/versionchecks.qml"));
-    QCOMPARE(QFileInfo(":/versionchecks.qml").size(), 0);
+    QVERIFY(QFile::exists(":/data/versionchecks.qml"));
+    QVERIFY(QFileInfo(":/data/versionchecks.qml").size() > 0);
 
     Q_ASSERT(!temporaryModifiedCachedUnit);
     QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
-    const QV4::CompiledData::Unit *originalUnit = QQmlMetaType::findCachedCompilationUnit(QUrl("qrc:/versionchecks.qml"), &error);
+    const QQmlPrivate::CachedQmlUnit *originalUnit = QQmlMetaType::findCachedCompilationUnit(
+            QUrl("qrc:/data/versionchecks.qml"), &error);
     QVERIFY(originalUnit);
-    QV4::CompiledData::Unit *tweakedUnit = (QV4::CompiledData::Unit *)malloc(originalUnit->unitSize);
-    memcpy(reinterpret_cast<void *>(tweakedUnit), reinterpret_cast<const void *>(originalUnit), originalUnit->unitSize);
+    QV4::CompiledData::Unit *tweakedUnit = (QV4::CompiledData::Unit *)malloc(originalUnit->qmlData->unitSize);
+    memcpy(reinterpret_cast<void *>(tweakedUnit),
+           reinterpret_cast<const void *>(originalUnit->qmlData),
+           originalUnit->qmlData->unitSize);
     tweakedUnit->version = QV4_DATA_STRUCTURE_VERSION - 1;
     temporaryModifiedCachedUnit = new QQmlPrivate::CachedQmlUnit{tweakedUnit, nullptr, nullptr};
 
     auto testHandler = [](const QUrl &url) -> const QQmlPrivate::CachedQmlUnit * {
-        if (url == QUrl("qrc:/versionchecks.qml"))
+        if (url == QUrl("qrc:/data/versionchecks.qml"))
             return temporaryModifiedCachedUnit;
         return nullptr;
     };
@@ -373,15 +390,14 @@ void tst_qmlcachegen::versionChecksForAheadOfTimeUnits()
 
     {
         QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
-        QVERIFY(!QQmlMetaType::findCachedCompilationUnit(QUrl("qrc:/versionchecks.qml"), &error));
+        QVERIFY(!QQmlMetaType::findCachedCompilationUnit(QUrl("qrc:/data/versionchecks.qml"), &error));
         QCOMPARE(error, QQmlMetaType::CachedUnitLookupError::VersionMismatch);
     }
 
     {
         QQmlEngine engine;
-        QQmlComponent component(&engine, QUrl("qrc:/versionchecks.qml"));
-        QCOMPARE(component.status(), QQmlComponent::Error);
-        QCOMPARE(component.errorString(), QString("qrc:/versionchecks.qml:-1 File was compiled ahead of time with an incompatible version of Qt and the original file cannot be found. Please recompile\n"));
+        CleanlyLoadingComponent component(&engine, QUrl("qrc:/data/versionchecks.qml"));
+        QCOMPARE(component.status(), QQmlComponent::Ready);
     }
 
     Q_ASSERT(temporaryModifiedCachedUnit);
@@ -392,14 +408,34 @@ void tst_qmlcachegen::versionChecksForAheadOfTimeUnits()
     QQmlMetaType::removeCachedUnitLookupFunction(testHandler);
 }
 
+void tst_qmlcachegen::retainedResources()
+{
+    QFile file(":/Retain.qml");
+    QVERIFY(file.open(QIODevice::ReadOnly));
+    QVERIFY(file.readAll().startsWith("import QtQml 2.0"));
+}
+
+void tst_qmlcachegen::skippedResources()
+{
+    QFile file(":/not/Skip.qml");
+    QVERIFY(file.open(QIODevice::ReadOnly));
+    QVERIFY(file.readAll().startsWith("import QtQml 2.0"));
+
+    QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
+    const QQmlPrivate::CachedQmlUnit *unit = QQmlMetaType::findCachedCompilationUnit(
+            QUrl("qrc:/not/Skip.qml"), &error);
+    QCOMPARE(unit, nullptr);
+    QCOMPARE(error, QQmlMetaType::CachedUnitLookupError::NoUnitFound);
+}
+
 void tst_qmlcachegen::workerScripts()
 {
-    QVERIFY(QFile::exists(":/workerscripts/worker.js"));
-    QVERIFY(QFile::exists(":/workerscripts/worker.qml"));
-    QCOMPARE(QFileInfo(":/workerscripts/worker.js").size(), 0);
+    QVERIFY(QFile::exists(":/workerscripts/data/worker.js"));
+    QVERIFY(QFile::exists(":/workerscripts/data/worker.qml"));
+    QVERIFY(QFileInfo(":/workerscripts/data/worker.js").size() > 0);
 
     QQmlEngine engine;
-    CleanlyLoadingComponent component(&engine, QUrl("qrc:///workerscripts/worker.qml"));
+    CleanlyLoadingComponent component(&engine, QUrl("qrc:///workerscripts/data/worker.qml"));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QTRY_VERIFY(obj->property("success").toBool());
@@ -485,7 +521,7 @@ void tst_qmlcachegen::trickyPaths()
 {
     QFETCH(QString, filePath);
     QVERIFY2(QFile::exists(filePath), qPrintable(filePath));
-    QCOMPARE(QFileInfo(filePath).size(), 0);
+    QVERIFY(QFileInfo(filePath).size() > 0);
     QQmlEngine engine;
     QQmlComponent component(&engine, QUrl("qrc" + filePath));
     QScopedPointer<QObject> obj(component.create());
@@ -496,7 +532,7 @@ void tst_qmlcachegen::trickyPaths()
 void tst_qmlcachegen::qrcScriptImport()
 {
     QQmlEngine engine;
-    CleanlyLoadingComponent component(&engine, QUrl("qrc:///jsimport.qml"));
+    CleanlyLoadingComponent component(&engine, QUrl("qrc:///data/jsimport.qml"));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QTRY_COMPARE(obj->property("value").toInt(), 42);
@@ -559,14 +595,14 @@ void tst_qmlcachegen::fsScriptImport()
 void tst_qmlcachegen::moduleScriptImport()
 {
     QQmlEngine engine;
-    CleanlyLoadingComponent component(&engine, QUrl("qrc:///jsmoduleimport.qml"));
+    CleanlyLoadingComponent component(&engine, QUrl("qrc:///data/jsmoduleimport.qml"));
     QVERIFY2(!component.isError(), qPrintable(component.errorString()));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QTRY_VERIFY(obj->property("ok").toBool());
 
-    QVERIFY(QFile::exists(":/script.mjs"));
-    QCOMPARE(QFileInfo(":/script.mjs").size(), 0);
+    QVERIFY(QFile::exists(":/data/script.mjs"));
+    QVERIFY(QFileInfo(":/data/script.mjs").size() > 0);
 
     {
         auto componentPrivate = QQmlComponentPrivate::get(&component);
@@ -579,17 +615,26 @@ void tst_qmlcachegen::moduleScriptImport()
         QVERIFY(unitData->flags & QV4::CompiledData::Unit::IsESModule);
 
         QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
-        const QV4::CompiledData::Unit *unitFromResources = QQmlMetaType::findCachedCompilationUnit(QUrl("qrc:/script.mjs"), &error);
+        const QQmlPrivate::CachedQmlUnit *unitFromResources = QQmlMetaType::findCachedCompilationUnit(
+                QUrl("qrc:/data/script.mjs"), &error);
         QVERIFY(unitFromResources);
 
-        QCOMPARE(unitFromResources, compilationUnit->unitData());
+        QCOMPARE(unitFromResources->qmlData, compilationUnit->unitData());
     }
 }
 
+void tst_qmlcachegen::esModulesViaQJSEngine()
+{
+    QJSEngine engine;
+    QJSValue module = engine.importModule(":/data/module.mjs");
+    QJSValue result = module.property("entry").call();
+    QCOMPARE(result.toString(), "ok");
+}
+
 void tst_qmlcachegen::enums()
 {
     QQmlEngine engine;
-    CleanlyLoadingComponent component(&engine, QUrl("qrc:///Enums.qml"));
+    CleanlyLoadingComponent component(&engine, QUrl("qrc:///data/Enums.qml"));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QTRY_COMPARE(obj->property("value").toInt(), 200);
@@ -597,14 +642,78 @@ void tst_qmlcachegen::enums()
 
 void tst_qmlcachegen::sourceFileIndices()
 {
-    QVERIFY(QFile::exists(":/versionchecks.qml"));
-    QCOMPARE(QFileInfo(":/versionchecks.qml").size(), 0);
+    QVERIFY(QFile::exists(":/data/versionchecks.qml"));
+    QVERIFY(QFileInfo(":/data/versionchecks.qml").size() > 0);
 
     QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
-    const QV4::CompiledData::Unit *unitFromResources = QQmlMetaType::findCachedCompilationUnit(QUrl("qrc:/versionchecks.qml"), &error);
+    const QQmlPrivate::CachedQmlUnit *unitFromResources = QQmlMetaType::findCachedCompilationUnit(
+            QUrl("qrc:/data/versionchecks.qml"), &error);
     QVERIFY(unitFromResources);
-    QVERIFY(unitFromResources->flags & QV4::CompiledData::Unit::PendingTypeCompilation);
-    QCOMPARE(uint(unitFromResources->sourceFileIndex), uint(0));
+    QVERIFY(unitFromResources->qmlData->flags & QV4::CompiledData::Unit::PendingTypeCompilation);
+    QCOMPARE(uint(unitFromResources->qmlData->sourceFileIndex), uint(0));
+}
+
+void tst_qmlcachegen::reproducibleCache_data()
+{
+    QTest::addColumn<QString>("filePath");
+
+    QDir dir(dataDirectory());
+    for (const QString &entry : dir.entryList((QStringList() << "*.qml" << "*.js" << "*.mjs"), QDir::Files)) {
+        QTest::newRow(entry.toUtf8().constData()) << dir.filePath(entry);
+    }
+}
+
+void tst_qmlcachegen::reproducibleCache()
+{
+    QFETCH(QString, filePath);
+
+    QFile file(filePath);
+    QVERIFY(file.exists());
+
+    auto generate = [](const QString &path) {
+        if (!generateCache(path))
+            return QByteArray();
+        QFile generated(path + 'c');
+        [&](){ QVERIFY(generated.open(QIODevice::ReadOnly)); }();
+        const QByteArray result = generated.readAll();
+        generated.remove();
+        return result;
+    };
+
+    const QByteArray contents1 = generate(file.fileName());
+    const QByteArray contents2 = generate(file.fileName());
+    QCOMPARE(contents1, contents2);
+}
+
+void tst_qmlcachegen::parameterAdjustment()
+{
+    QQmlEngine engine;
+    CleanlyLoadingComponent component(&engine, QUrl("qrc:///data/parameterAdjustment.qml"));
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY(!obj.isNull()); // Doesn't crash
+}
+
+
+void tst_qmlcachegen::inlineComponent()
+{
+    bool ok = generateCache(testFile("inlineComponentWithId.qml"));
+    QVERIFY(ok);
+    QQmlEngine engine;
+    CleanlyLoadingComponent component(&engine, testFileUrl("inlineComponentWithId.qml"));
+    QTest::ignoreMessage(QtMsgType::QtInfoMsg, "42");
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY(!obj.isNull());
+}
+
+void tst_qmlcachegen::posthocRequired()
+{
+    bool ok = generateCache(testFile("posthocrequired.qml"));
+    QVERIFY(ok);
+    QQmlEngine engine;
+    CleanlyLoadingComponent component(&engine, testFileUrl("posthocrequired.qml"));
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY(obj.isNull() && component.isError());
+    QVERIFY(component.errorString().contains(QStringLiteral("Required property x was not initialized")));
 }
 
 QTEST_GUILESS_MAIN(tst_qmlcachegen)
