@@ -29,26 +29,31 @@
 #include <QtTest/QtTest>
 #include <QtTest/QSignalSpy>
 #include <QtGui/QStyleHints>
+#include <QtGui/private/qevent_p.h>
 #include <qpa/qwindowsysteminterface.h>
-#include <private/qquickpinchhandler_p.h>
+#include <QtQuick/private/qquickpinchhandler_p.h>
 #include <QtQuick/private/qquickrectangle_p.h>
 #include <QtQuick/qquickview.h>
 #include <QtQml/qqmlcontext.h>
 #include "../../../shared/util.h"
 #include "../../shared/viewtestutil.h"
 
+Q_LOGGING_CATEGORY(lcPointerTests, "qt.quick.pointer.tests")
+
 class tst_QQuickPinchHandler: public QQmlDataTest
 {
     Q_OBJECT
 public:
-    tst_QQuickPinchHandler() : device(0) { }
+    tst_QQuickPinchHandler() { }
 private slots:
-    void initTestCase();
     void cleanupTestCase();
     void pinchProperties();
     void scale();
     void scaleThreeFingers();
+    void scaleNativeGesture();
     void pan();
+    void dragAxesEnabled_data();
+    void dragAxesEnabled();
     void retouch();
     void cancel();
     void transformedpinchHandler_data();
@@ -56,17 +61,8 @@ private slots:
 
 private:
     QQuickView *createView();
-    QTouchDevice *device;
+    QPointingDevice *device = QTest::createTouchDevice();
 };
-void tst_QQuickPinchHandler::initTestCase()
-{
-    QQmlDataTest::initTestCase();
-    if (!device) {
-        device = new QTouchDevice;
-        device->setType(QTouchDevice::TouchScreen);
-        QWindowSystemInterface::registerTouchDevice(device);
-    }
-}
 
 void tst_QQuickPinchHandler::cleanupTestCase()
 {
@@ -192,12 +188,12 @@ void tst_QQuickPinchHandler::pinchProperties()
     QCOMPARE(rotMaxSpy.count(),1);
 }
 
-QTouchEvent::TouchPoint makeTouchPoint(int id, QPoint p, QQuickView *v, QQuickItem *i)
+QMutableEventPoint makeTouchPoint(int id, QPoint p, QQuickView *v, QQuickItem *i)
 {
-    QTouchEvent::TouchPoint touchPoint(id);
-    touchPoint.setPos(i->mapFromScene(p));
-    touchPoint.setScreenPos(v->mapToGlobal(p));
-    touchPoint.setScenePos(p);
+    QMutableEventPoint touchPoint(id);
+    touchPoint.setPosition(i->mapFromScene(p));
+    touchPoint.setGlobalPosition(v->mapToGlobal(p));
+    touchPoint.setScenePosition(p);
     return touchPoint;
 }
 
@@ -358,6 +354,79 @@ void tst_QQuickPinchHandler::scaleThreeFingers()
     QCOMPARE(pinchHandler->active(), false);
 }
 
+void tst_QQuickPinchHandler::scaleNativeGesture()
+{
+    QQuickView *window = createView();
+    QScopedPointer<QQuickView> scope(window);
+    window->setSource(testFileUrl("pinchproperties.qml"));
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+    QVERIFY(window->rootObject() != nullptr);
+    qApp->processEvents();
+
+    QQuickPinchHandler *pinchHandler = window->rootObject()->findChild<QQuickPinchHandler*>("pinchHandler");
+    QVERIFY(pinchHandler != nullptr);
+    QQuickItem *root = qobject_cast<QQuickItem*>(window->rootObject());
+    QVERIFY(root != nullptr);
+    QQuickItem *target = window->rootObject()->findChild<QQuickItem*>("blackrect");
+    QVERIFY(target != nullptr);
+
+    QPointF targetPos = target->position();
+    ulong ts = 1;
+
+    // first pinch: scale it up
+    const qreal expectedScale = 1.1;
+    QPointF pinchPos(75, 75);
+    QPointF pinchLocalPos = target->mapFromScene(pinchPos);
+    // target position is adjusted in QQuickItemPrivate::adjustedPosForTransform()
+    // so as to compensate for the change in size, to hold the centroid in place
+    const QPointF expectedPos = targetPos + QPointF( (pinchPos.x() - target->x()) * (expectedScale - 1),
+                                                     (pinchPos.y() - target->y()) * (expectedScale - 1) );
+    QWindowSystemInterface::handleGestureEvent(window, ts++, QPointingDevice::primaryPointingDevice(),
+                                               Qt::BeginNativeGesture, pinchPos, pinchPos);
+    QWindowSystemInterface::handleGestureEventWithRealValue(window, ts++, QPointingDevice::primaryPointingDevice(),
+                                                            Qt::ZoomNativeGesture, expectedScale - 1, pinchPos, pinchPos);
+    QTRY_COMPARE(target->scale(), expectedScale);
+    QCOMPARE(pinchHandler->active(), true);
+    QCOMPARE(pinchHandler->centroid().position(), pinchLocalPos);
+    QCOMPARE(pinchHandler->centroid().scenePosition(), pinchPos);
+    QVERIFY(qAbs(target->position().x() - expectedPos.x()) < 0.001);
+    QVERIFY(qAbs(target->position().y() - expectedPos.y()) < 0.001);
+    QCOMPARE(pinchHandler->scale(), expectedScale);
+    QCOMPARE(pinchHandler->activeScale(), expectedScale);
+    QCOMPARE(pinchHandler->translation(), QVector2D());
+    QCOMPARE(pinchHandler->rotation(), 0);
+    QWindowSystemInterface::handleGestureEvent(window, ts++, QPointingDevice::primaryPointingDevice(),
+                                               Qt::EndNativeGesture, pinchPos, pinchPos);
+    QTRY_COMPARE(pinchHandler->active(), false);
+    QCOMPARE(target->scale(), expectedScale);
+    QCOMPARE(pinchHandler->scale(), expectedScale);
+    QCOMPARE(pinchHandler->activeScale(), 1);
+    QCOMPARE(pinchHandler->translation(), QVector2D());
+    QCOMPARE(pinchHandler->rotation(), 0);
+
+    // second pinch at a different position: scale it down to original size again
+    const qreal reverseScale = (1 / expectedScale);
+    pinchPos = QPointF(125, 125);
+    pinchLocalPos = target->mapFromScene(pinchPos);
+    QWindowSystemInterface::handleGestureEvent(window, ts++, QPointingDevice::primaryPointingDevice(),
+                                               Qt::BeginNativeGesture, pinchPos, pinchPos);
+    QWindowSystemInterface::handleGestureEventWithRealValue(window, ts++, QPointingDevice::primaryPointingDevice(),
+                                                            Qt::ZoomNativeGesture, reverseScale - 1, pinchPos, pinchPos);
+    QTRY_COMPARE(target->scale(), 1);
+    QCOMPARE(pinchHandler->active(), true);
+    QCOMPARE(pinchHandler->centroid().position(), pinchLocalPos);
+    QCOMPARE(pinchHandler->centroid().scenePosition(), pinchPos);
+    QCOMPARE(pinchHandler->scale(), 1);
+    QCOMPARE(pinchHandler->activeScale(), reverseScale);
+    QWindowSystemInterface::handleGestureEvent(window, ts++, QPointingDevice::primaryPointingDevice(),
+                                               Qt::EndNativeGesture, pinchPos, pinchPos);
+    QTRY_COMPARE(pinchHandler->active(), false);
+    QCOMPARE(target->scale(), 1);
+    QCOMPARE(pinchHandler->scale(), 1);
+    QCOMPARE(pinchHandler->activeScale(), 1);
+}
+
 void tst_QQuickPinchHandler::pan()
 {
     QQuickView *window = createView();
@@ -459,6 +528,82 @@ void tst_QQuickPinchHandler::pan()
     QVERIFY(!root->property("pinchActive").toBool());
 }
 
+void tst_QQuickPinchHandler::dragAxesEnabled_data()
+{
+    QTest::addColumn<bool>("xEnabled");
+    QTest::addColumn<bool>("yEnabled");
+
+    QTest::newRow("both enabled") << true << true;
+    QTest::newRow("x enabled") << true << false;
+    QTest::newRow("y enabled") << false << true;
+    QTest::newRow("both disabled") << false << false;
+}
+
+void tst_QQuickPinchHandler::dragAxesEnabled()
+{
+    QQuickView *window = createView();
+    QScopedPointer<QQuickView> scope(window);
+    window->setSource(testFileUrl("pinchproperties.qml"));
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+    QVERIFY(window->rootObject() != nullptr);
+    QQuickItem *blackRect = window->rootObject()->findChild<QQuickItem*>("blackrect");
+    QVERIFY(blackRect != nullptr);
+    QQuickPinchHandler *pinchHandler = blackRect->findChild<QQuickPinchHandler*>();
+    QVERIFY(pinchHandler != nullptr);
+
+    QFETCH(bool, xEnabled);
+    QFETCH(bool, yEnabled);
+    const int dragThreshold = QGuiApplication::styleHints()->startDragDistance();
+    pinchHandler->xAxis()->setEnabled(xEnabled);
+    pinchHandler->yAxis()->setEnabled(yEnabled);
+    QPoint c = blackRect->mapToScene(blackRect->clipRect().center()).toPoint();
+    QPoint p0 = c - QPoint(0, dragThreshold);
+    QPoint p1 = c + QPoint(0, dragThreshold);
+    QPoint blackRectPos = blackRect->position().toPoint();
+
+    // press two points, one above the rectangle's center and one below
+    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(window, device);
+    pinchSequence.press(0, p0, window).press(1, p1, window).commit();
+    QQuickTouchUtils::flush(window);
+
+    // expand the pinch vertically
+    p0 -= QPoint(0, dragThreshold);
+    p1 += QPoint(0, dragThreshold);
+    pinchSequence.move(0, p0, window).move(1, p1, window).commit();
+    for (int pi = 0; pi < 4; ++pi) {
+        p0 -= QPoint(0, dragThreshold);
+        p1 += QPoint(0, dragThreshold);
+        pinchSequence.move(0, p0, window).move(1, p1, window).commit();
+        QQuickTouchUtils::flush(window);
+        qCDebug(lcPointerTests) << pi << "active" << pinchHandler->active() << "pts" << p0 << p1
+                                << "centroid" << pinchHandler->centroid().scenePosition()
+                                << "rect pos" << blackRect->position() << "scale" << blackRect->scale();
+    }
+    QCOMPARE(pinchHandler->active(), true);
+    QVERIFY(blackRect->scale() >= 2.0);
+    // drag started, but we only did scaling without any translation
+    QCOMPARE(pinchHandler->centroid().scenePosition().toPoint(), c);
+    QCOMPARE(blackRect->position().toPoint().x(), blackRectPos.x());
+    QCOMPARE(blackRect->position().toPoint().y(), blackRectPos.y());
+
+    // drag diagonally
+    p0 += QPoint(150, 150);
+    p1 += QPoint(150, 150);
+    pinchSequence.move(0, p0, window).move(1, p1, window).commit();
+    QQuickTouchUtils::flush(window);
+    // the target should move if the xAxis is enabled, or stay in place if not
+    qCDebug(lcPointerTests) << "after diagonal drag: pts" << p0 << p1
+                            << "centroid" << pinchHandler->centroid().scenePosition()
+                            << "rect pos" << blackRect->position() << "scale" << blackRect->scale();
+    QCOMPARE(pinchHandler->centroid().scenePosition().toPoint(), QPoint(250, 250));
+    QCOMPARE(blackRect->position().toPoint().x(), xEnabled ? 140 : blackRectPos.x()); // because of xAxis.maximum
+    QCOMPARE(blackRect->position().toPoint().y(), yEnabled ? 170 : blackRectPos.y()); // because of yAxis.maximum
+
+    QTest::touchEvent(window, device).release(0, p0, window).release(1, p1, window);
+    QQuickTouchUtils::flush(window);
+}
+
 // test pinchHandler, release one point, touch again to continue pinchHandler
 void tst_QQuickPinchHandler::retouch()
 {
@@ -597,8 +742,7 @@ void tst_QQuickPinchHandler::cancel()
 
         QSKIP("cancel is not supported atm");
 
-        QTouchEvent cancelEvent(QEvent::TouchCancel);
-        cancelEvent.setDevice(device);
+        QTouchEvent cancelEvent(QEvent::TouchCancel, device);
         QCoreApplication::sendEvent(window, &cancelEvent);
         QQuickTouchUtils::flush(window);
 
